;*********************************************************************
; Interrupt handler
; Scelta dell'interrupt che si  generato (tim0_int oppure rx_int)

; Viene verificato se  stato generato l'interrupt rx_int ( stato ricevuto
; almeno un carattere dalla Usart ed il registro-buffer RCREG non  vuoto)
; in due momenti distinti:
; - Se  stato generato un interrupt dalla Usart (ricevuto carattere)
; - Al termine di rx_int (nel caso che il registro RCREG contiene pi caratteri)

        ORG     0x04
        movwf   W_Temp          ;Salvataggio registro w in W_Temp
        swapf   STATUS,w        ;Salvataggio registro STATUS in St_Temp (include banchi)
        movwf   St_Temp
        movf    FSR,w           ;Salvataggio registro FSR in FsrTemp
        movwf   FsrTemp

        BANK0                   ;Seleziona banchi ram
;        IBANK01
        btfsc   INTCON,T0IF     ;Verifica la generazione dell'interrupt Timer0
        goto    tim0_int        ;Interrupt Timer0
        call    rx_int          ;Interrupt Rx Usart

end_ih                          ;Fine Interrupt
        movf    FsrTemp,w       ;Ripristino registro FSR
        movwf   FSR
        swapf   St_Temp,w       ;Ripristino registro STATUS (e ripristino banchi)
        movwf   STATUS
        swapf   W_Temp,f        ;Ripristino registro w
        swapf   W_Temp,w
        retfie


;-----------------------------------------------------
;*** Interrupt Timer0 ***
;E' eseguito ogni 10ms e gestisce la matrice di ingresso e il multiplex dei 2 display

tim0_int                        ;Ricarica TMR0 per eseguire il prossimo timer
        movlw   D'60'           ;(256-60) * 51us ==> 9996us = 10ms
        movwf   TMR0

        btfss   F_DIGIT         ;Inverte flag per mux digit display
        goto    t0_1
        bcf     F_DIGIT
        goto    t0_2
t0_1    bsf     F_DIGIT

t0_2    incf    CBlink,f        ;Inverte flag per lampeggio display ogni 100ms
        movf    CBlink,w
        xorlw   D'10'
        btfss   STATUS,Z
        goto    t0_4
        clrf    CBlink
        btfss   F_BLINK
        goto    t0_3
        bcf     F_BLINK
        goto    t0_4
t0_3    bsf     F_BLINK

t0_4    btfss   F_DIGBLK        ;Se richiesto da flag, genera lampeggio display
        goto    t0_5
        btfsc   F_BLINK
        goto    t0_5
        clrf    SrData1
        goto    t0_6

t0_5    movlw   0x07            ;Carica segmenti display
        movwf   PCLATH
        movf    DigitH,w
        btfss   F_DIGIT
        movf    DigitL,w
        call    TBL_7SEGMENT_DISPLAY
        movwf   SrData1
        btfss   F_HBEAT         ;Pilota punto su digit Lsb secondo HeartBeat Main
        goto    t0_6
        btfss   F_DIGIT
        bsf     SrData1,2

t0_6    movlw   0x07            ;Carica abilitazione linea matrice di ingresso
        movwf   PCLATH
        movf    CMaLine,w
        call    TBL_MAINPUT_LINE_ENABLE
        movwf   SrData2

        btfss   F_DIGIT         ;Accende catodo display H oppure L secondo flag
        goto    t0_7
        bsf     SrData2,0
        goto    t0_8
t0_7    bsf     SrData2,1

t0_8    call    send_shift_register            ;Invia shift register

t0_e    bcf     INTCON,T0IF     ;Riabilita interrupt su TMR0
        goto    end_ih


;---------------------------------------------------------------------------------
;Interrupt su Rx carattere da Usart
;Eseguito ogni volta che la Usart ha ricevuto un carattere
;Se modulo VS1000, setta i corrspondenti 4 Flag se sono state ricevute le seguenti
;sequenze di caratteri: SD, !SD, play, done
;Se modulo MDFLY, setta i 2 flag se ricevuti i relativi caratteri

rx_int  btfss   PIR1,RCIF       ;Verifica se irq generato da Usart ed il buffer non  vuoto
        return                  ;Ripete la sub finch il buffer non  vuoto

        movf    RCREG,w         ;Preleva carattere ricevuto e lo copia su appoggio
        movwf   ChrRx

        btfss   RCSTA,OERR      ;Se errore OverRun, ri-abilita Usart per cancellare errore
        goto    rx_0
        bcf     RCSTA,CREN
        nop
        bsf     RCSTA,CREN

rx_0    btfss   F_MODTYPE       ;Salta se selezionato modulo MDFLY
        goto    rx_7


;Gestione modulo VS1000
rx_1    movlw   0x07            ;Preleva carattere in tabella per confronto
        movwf   PCLATH          ;alla posizione del puntatore
        movf    PTblRxStr,w
        call    TBL_RX_STRING
        movwf   TempI
        movf    TempI,w
        btfsc   STATUS,Z        ;Esce se carattere tappo su tabella (0)
        goto    rx_6
        xorwf   ChrRx,w         ;Confronta con carattere ricevuto
        btfsc   STATUS,Z
        goto    rx_2
        movf    CRxByte,w       ;Se diverso e nessun carattere ricevuto uguale,
        btfss   STATUS,Z        ;preleva lunghezza stringa per individuare il primo
        goto    rx_6            ;carattere della stringa successiva
;        movlw   0x07
;        movwf   PCLATH
        movf    CRxStr,w
        call    TBL_RX_STRING_LEN
        addwf   PTblRxStr,f
        incf    CRxStr,f
        goto    rx_1

rx_2    incf    CRxByte,f       ;Incrementa contatore caratteri uguali
;        movlw   0x07           ;e verifica se tutti i caratteri della stringa
;        movwf   PCLATH         ;sono stati ricevuti in sequenza
        movf    CRxStr,w
        call    TBL_RX_STRING_LEN
        xorwf   CRxByte,w
        btfsc   STATUS,Z
        goto    rx_3
        incf    PTblRxStr,f
        goto    rx_int

rx_3    movlw   B'10000000'     ;Se tutti i caratteri sono stati ricevuti setta Flag
        movwf   TempI           ;interessato lasciando inalterati gli altri Flag
rx_4    movf    CRxStr,w
        btfsc   STATUS,Z
        goto    rx_5
        bcf     STATUS,C
        rrf     TempI,f
        decf    CRxStr,f
        goto    rx_4
rx_5    movf    FlagsRx,w
        iorwf   TempI,w
        movwf   FlagsRx

rx_6    clrf    PTblRxStr       ;Azzera puntatori e variabili contatori
        clrf    CRxByte
        clrf    CRxStr
        goto    rx_int


;Gestione modulo MDFLY
rx_7    movf    ChrRx,w         ;Setta flag STOP se ricevuti caratteri 0xEF o 0xFF
        xorlw   0xEF
        btfsc   STATUS,Z
        goto    rx_8
        movf    ChrRx,w
        xorlw   0xFF
        btfss   STATUS,Z
        goto    rx_9
rx_8    bsf     F_RX_STOP
        goto    rx_int

rx_9    movf    ChrRx,w         ;Setta flag PLAY se ricevuto lo stesso carattere
        xorwf   ChrTrk,w        ;trasmesso, corrispondente al numero traccia
        btfsc   STATUS,Z
        bsf     F_RX_PLAY
        goto    rx_int