;************************************************************************
; Music_Box.ASM Rev.1
; 15/2/2014 by Sasa

; Generatore di suoni 30 ingressi con modulo VS1000
; Maggiori informazioni nel file Music_Box.PDF

;************************************************************************

        PROCESSOR       16F628                  ;Tipo di processore
        INCLUDE         "P16F628.INC"           ;inclusione File .inc
        __CONFIG        B'11110101000110'       ;Nessuna protezione codice
                                                ;Disabilitato LVP
                                                ;Brown-out abilitato
                                                ;Funzione MCLR non attiva
                                                ;Power Up abilitato
                                                ;Watch-dog abilitato
                                                ;Oscillatore esterno HS (20 MHz)


;--- Dichiarazione costanti I/O
#DEFINE I_MIN_C1  PORTA,5       ;Ingresso matrice Colonna 1
#DEFINE I_MIN_C2  PORTA,4       ;Ingresso matrice Colonna 2
#DEFINE I_MIN_C3  PORTA,3       ;Ingresso matrice Colonna 3
#DEFINE I_MIN_C4  PORTA,2       ;Ingresso matrice Colonna 4
#DEFINE I_MIN_C5  PORTA,1       ;Ingresso matrice Colonna 5
#DEFINE O_LED_HB  PORTA,0       ;Uscita led HeartBeat

#DEFINE I_PPLAYN  PORTB,0       ;Ingresso Pulsante Play/Next
#DEFINE I_PSTOP   PORTB,4       ;Ingresso Pulsante Stop
#DEFINE I_MODTYPE PORTB,3       ;Ingresso Jumper Modulo (1=VS1000, 0=MDFLY)
#DEFINE O_SR_DATA PORTB,7       ;Uscita Data Shift Register
#DEFINE O_SR_CLK  PORTB,5       ;Uscita Clock Shift Register
#DEFINE O_SR_STRB PORTB,6       ;Uscita Strobe Shift Register

#DEFINE I_RX      PORTB,1       ;Ingresso Seriale dati
#DEFINE O_TX      PORTB,2       ;Uscita Seriale dati


;--- Definizione Flag per controllo ricezione stringhe di caratteri dal modulo
#define F_RX_SD   FlagsRx,7     ;Ricevuta sequenza di caratteri 'SD'
#define F_RX_NOSD FlagsRx,6     ;Ricevuto sequenza di caratteri '!SD'
#define F_RX_PLAY FlagsRx,5     ;Ricevuto sequenza 'play' o carattere play
#define F_RX_STOP FlagsRx,4     ;Ricevuto sequenza 'done' o carattere stop


;--- Dichiarazione Flag per utilizzi generici
#define F_DIGIT   Flags,7       ;Scelta digit display per multiplexer (1=Hi, 0=Lo)
#define F_BLINK   Flags,6       ;Lampeggio display
#define F_DIGBLK  Flags,5       ;Abilitazione lampeggio display
#define F_MODTYPE Flags,4       ;Tipo di modulo utilizzato (1=VS1000, 0=MDFLY)
#define F_HBEAT   Flags,3       ;HeartBeat mainloop
#define F_CMD_EXE Flags,2       ;Comando Play da utente
#define F_AUTPLAY Flags,1       ;Modalit Play sequenziale attivata da utente


;--- Dichiarazione costanti
TIMEOUT   equ     D'100'        ;Timeout esecuzione comandi (50 * 0.1s = 5s)


;--- Dichiarazione variabili Ram (banco 0)
          org     0x20
ModStat   res     0x01          ;Stato modulo esterno VS1000

CntPlay   res     0x01          ;Contatore riproduzione traccia
CTimeout  res     0x01          ;Contatore timeout esecuzione comandi

MaInput1  res     0x01          ;Matrice Ingressi 1..5
MaInput2  res     0x01          ;Matrice Ingressi 6..10
MaInput3  res     0x01          ;Matrice Ingressi 11..15
MaInput4  res     0x01          ;Matrice Ingressi 16..20
MaInput5  res     0x01          ;Matrice Ingressi 21..25
MaInput6  res     0x01          ;Matrice Ingressi 26..30
MaInPrev1 res     0x01          ;Matrice Ingressi Stati precedenti 1..5
MaInPrev2 res     0x01          ;Matrice Ingressi Stati precedenti 6..10
MaInPrev3 res     0x01          ;Matrice Ingressi Stati precedenti 11..15
MaInPrev4 res     0x01          ;Matrice Ingressi Stati precedenti 16..20
MaInPrev5 res     0x01          ;Matrice Ingressi Stati precedenti 21..25
MaInPrev6 res     0x01          ;Matrice Ingressi Stati precedenti 26..30
MaInEdge  res     0x01          ;Numero ingresso 1..30 su cui  stato rilevato fronte 0->1
                                ;(1..30 = rilevato fronte, 0 = nessun fronte rilevato)
CMaInput  res     0x01          ;Contatore ingresso per confronto
CMaLine   res     0x01          ;Contatore per abilitazione riga matrice
CMaDelay  res     0x01          ;Ritardo esecuzione routine matrice
CMaByte   res     0x01          ;Contatore byte
CMaBit    res     0x01          ;Contatore bit

DigitH    res     0x01          ;Valori associati alla singola cifra display
DigitL    res     0x01          ;per prelievo da tabella
CBlink    res     0x01          ;Contatore lampeggio display

SrData1   res     0x01          ;Byte da inviare sullo shift register
SrData2   res     0x01
CSrByte   res     0x01          ;Contatori per invio byte su shift-register
CSrBit    res     0x01

DigD_H    res     0x01          ;Cifre per conversione binario-decimale
DigD_L    res     0x01

ChrTrk    res     0x01          ;Carattere traccia tx per confronto play modulo MDFLY

ChrRx     res     0x01          ;Carattere ricevuto
PTblRxStr res     0x01          ;Puntatore per tabella verifica caratteri ricevuti
CRxStr    res     0x01          ;Contatore per confronto stringa ricevuta
CRxByte   res     0x01          ;Contatore per confronto byte rx all'interno della stringa

Count1    res     0x01          ;Contatore generico in main
CntHbLed  res     0x01          ;Contatore per lampeggio Led HeartBeat

CntDel1   res     0x01          ;Contatori per ritardo in main
CntDel2   res     0x01

FlagsRx	  res     0x01          ;Flag per stringhe ricevute dalla porta seriale (vedi #define)
Flags	  res     0x01          ;Flag per utilizzi generici (vedi #define)

TempM     res     0x01          ;Appoggio temporaneo in Main
TempI     res     0x01          ;Appoggio temporaneo in Interrupt


;--- Dichiarazioni Variabili Ram (comuni a tutti e 4 i banchi)
          org     0x70
W_Temp    res     0x01          ;Salvataggio w durante interrupt
St_Temp   res     0x01          ;Salvataggio STATUS durante interrupt
FsrTemp   res     0x01          ;Salvataggio FSR durante interrupt


;*********************************************************************
;Definizione Macro per selezionare i 3 banchi di memoria Ram
BANK0	MACRO                   ;Indirizzamento diretto
	bcf     STATUS,RP1
	bcf     STATUS,RP0
        ENDM
BANK1	MACRO
	bcf     STATUS,RP1
	bsf     STATUS,RP0
	ENDM
BANK2	MACRO
	bsf     STATUS,RP1
	bcf     STATUS,RP0
	ENDM

IBANK01	MACRO                   ;Indirizzamento indiretto
	bcf    STATUS,IRP
	ENDM
IBANK23	MACRO
	bsf    STATUS,IRP
	ENDM


;************************************************************************
;Inizio memoria programma (locazione 00)
        ORG     0x00
        goto    main_start


;************************************************************************
;Inclusione routine per gestioni varie
        INCLUDE "IRQS.INC"      ;Interrupt
        INCLUDE "SUBS.INC"      ;Subroutine varie
        INCLUDE "MOD_SUBS.INC"  ;Subroutine gestione moduli


;************************************************************************
;Programma PRINCIPALE

;Configurazione porte I/O (vedi #DEFINE all'inizio) e settaggi vari (1=IN 0=OUT)
main_start
        BANK0                   ;Ram Banco 0
        clrf    PORTA
        clrf    PORTB
        movlw   0x07            ;Abilita i pin RA0..RA3 come I/O standard
        movwf   CMCON
        BANK1                   ;Ram Banco 1
        movlw   B'00111110'     ;Configurazione Porta A
        movwf   TRISA
        movlw   B'00011011'     ;Configurazione Porta B
        movwf   TRISB
        movlw   B'10000111'     ;bit7: Disattiva circuiteria weak-up ingressi
        movwf   OPTION_REG      ;bit5: Ingresso prescaler a fOsc/4 (fOsc = 20MHz)
                                ;bit3: Ingresso registro TMR0 a Prescaler
                                ;bit2,1,0: Divisione prescaler 1:256
                                ;Frequenza uscita prescaler = 19531 Hz

;Azzera variabili e flag
        BANK0
        clrf    FlagsRx         ;Azzera tutti i flag di stato
        clrf    Flags
        clrf    ModStat         ;Azzera stato modulo e contatori
        clrf    CntPlay
        clrf    CBlink
        clrf    CntHbLed

        movlw   D'10'           ;Spegne tutti i segmenti dei 2 digit del display
        movwf   DigitH
        movwf   DigitL

        clrf    CMaDelay        ;Azzera contatori associati alla matrice di ingresso
        clrf    CMaLine

        clrf    PTblRxStr       ;Azzera variabili associate alla ricezione caratteri
        clrf    CRxStr
        clrf    CRxByte
        clrf    ChrRx

;Scelta modulo: Jumper aperto per VS1000, Jumper chiuso per MDFLY
        bcf     F_MODTYPE
        btfsc   I_MODTYPE       ;Copia ingresso jumper modulo su flag 
        bsf     F_MODTYPE       ;per evitare possibili incongruenze

;Inizializzazione Usart 115200/9600,n,8,1 @20MHz con interrupt su Rx
        movlw   D'10'           ;Fattore di divisione 115200 bps con BRGH=1 (VS1000)
        btfss   F_MODTYPE
        movlw   D'129'          ;Fattore di divisione 9600 bps con BRGH=1 (MDFLY)
        BANK1
        movwf   SPBRG
        movlw   B'00100100'     ;Registro Stato trasmissione e controllo
        movwf   TXSTA
        movlw   B'00100000'     ;PIE1 bit 5: Interrupt Rx Usart abilitato
        movwf   PIE1
        BANK0
        movlw   B'10010000'     ;Registro Stato ricezione e controllo
        movwf   RCSTA

;Abilita irq su Timer0 (refresh display) e su Usart (ricezione caratteri dalla porta seriale)
        movlw   B'11100000'     ;bit7=abilitazione globale interrupt
        movwf   INTCON          ;bit6=abilitazione interrupt da periferiche
                                ;bit5=abilitazione interrupt timer0

;Se premuti pulsanti play e stop all'accensione, esegue una delle 3 routine di test
        call    delay_100ms
        btfss   I_PPLAYN
        goto    ml_1
        btfss   I_PSTOP
        goto    main_test2      ;Esegue Test2 se premuto pulsante STOP
        goto    ml_2
ml_1    btfss   I_PSTOP
        goto    main_test3      ;Esegue Test3 se premuti pulsanti PLAY + STOP
        goto    main_test1      ;Esegue Test1 se premuto pulsante PLAY

;Inizializza il modulo precedentemente selezionato e le variabili di ingresso della matrice
ml_2    call    init_module
        call    init_matrix_input

;*** Main loop controllo ingressi e pulsanti
main_loop_input
        call    read_matrix_input_edge     ;Legge ingressi matrice
        movf    MaInEdge,w                 ;Se rilevato fronte 0->1 su ingressi imposta
        btfsc   STATUS,Z                   ;riproduzione traccia
        goto    mli_1
        movwf   CntPlay
        bsf     F_CMD_EXE
        bcf     F_AUTPLAY
        goto    main_loop_module

mli_1   btfsc   I_PPLAYN                   ;Verifica se premuto il pulsante PLAY
        goto    mli_5                      ;e attende il rilascio
mli_1a  call    delay_100ms
        btfss   I_PPLAYN
        goto    mli_1a
        movf    ModStat,w                  ;Se modulo in stato 1 (Ready),
        xorlw   0x01                       ;imposta esecuzione prima traccia
        btfss   STATUS,Z
        goto    mli_2
        movlw   0x01
        movwf   CntPlay
        goto    mli_4
mli_2   movf    ModStat,w                  ;Se modulo in stato 2 (WaitRxPlay),
        xorlw   0x02                       ;imposta esecuzione traccia successiva (fino a 30)
        btfss   STATUS,Z
        goto    mli_3
        movf    CntPlay,w
        xorlw   D'30'
        btfsc   STATUS,Z
        clrf    CntPlay
        incf    CntPlay,f
        goto    mli_4
mli_3   movf    ModStat,w                  ;Se modulo in stato 3 (Play),
        xorlw   0x03                       ;imposta esecuzione traccia successiva
        btfss   STATUS,Z
        goto    main_loop_module
mli_4   bsf     F_CMD_EXE                  ;Imposta flag per esecuzione traccia
        bsf     F_AUTPLAY
        goto    main_loop_module

mli_5   btfsc   I_PSTOP                    ;Verifica se premuto il pulsante STOP
        goto    main_loop_module           ;e attende il rilascio
mli_5a  call    delay_100ms
        btfss   I_PSTOP
        goto    mli_5a
        movf    ModStat,w                  ;Se modulo in stato 2 (WaitRxPlay),
        xorlw   0x02                       ;imposta lo stato 1 (Ready)
        btfss   STATUS,Z
        goto    mli_6
        movlw   0x01
        movwf   ModStat
        goto    main_loop_module
mli_6   movf    ModStat,w                  ;Se modulo in stato 3 (Play),
        xorlw   0x03                       ;imposta esecuzione comando di Stop
        btfss   STATUS,Z
        goto    main_loop_module
        bsf     F_CMD_EXE
        clrf    CntPlay
        bcf     F_AUTPLAY


;*** Main loop gestione modulo
main_loop_module
        movf    ModStat,w                  ;** Modulo in stato 1 (Ready)
        xorlw   0x01
        btfss   STATUS,Z
        goto    mlm2_1                     
        movlw   D'11'                      ;Visualizza sul display '--' fisso
        movwf   DigitH
        movwf   DigitL
        bcf     F_DIGBLK
mlm1_2  btfss   F_CMD_EXE                  ;Se ricevuto comando Play con CntPlay<>0
        goto    mlm1_3
        bcf     F_CMD_EXE
        movf    CntPlay,w
        btfsc   STATUS,Z
        goto    mlm1_3
        movf    CntPlay,w                  ;Converte numero traccia in due numeri decimali
        call    conv_dec_2digit            ;per invio comando al modulo
        call    send_cmd_play              ;e per visualizzazione su display
        movlw   TIMEOUT
        movwf   CTimeout
        movlw   0x02                       ;Imposta stato 2 (WaitRxPlay)
        movwf   ModStat
mlm1_3  btfss   F_RX_PLAY                  ;Se ricevuta risposta 'play'
        goto    mlm_e
        bcf     F_RX_PLAY
        movlw   0x03                       ;Imposta stato 3 (Play)
        movwf   ModStat
        goto    mlm_e

mlm2_1  movf    ModStat,w                  ;** Modulo in stato 2 (WaitRxPlay)
        xorlw   0x02
        btfss   STATUS,Z
        goto    mlm3_1
        movf    DigD_H,w                   ;Visualizza sul display Numero lampeggiante
        movwf   DigitH
        movf    DigD_L,w
        movwf   DigitL
        bsf     F_DIGBLK
        btfss   F_AUTPLAY                  ;Se comando Play manuale
        goto    mlm1_2                     ;decrementa timeout per saltare
        decfsz  CTimeout,f                 ;alla prossima traccia
        goto    mlm1_2
        goto    mlm3_3

mlm3_1  movf    ModStat,w                  ;** Modulo in stato 3 (Play)
        xorlw   0x03
        btfss   STATUS,Z
        goto    mlm4_1
        bcf     F_DIGBLK                   ;Visualizza sul display Numero fisso
        btfss   F_CMD_EXE                  ;Se comando utente Play, invia Stop al modulo
        goto    mlm3_2
        call    send_cmd_stop
        movlw   0x04                       ;Imposta stato 4 (WaitRxCancel)
        movwf   ModStat
mlm3_2  btfss   F_RX_STOP                  ;Se ricevuta risposta 'stop'
        goto    mlm_e
        bcf     F_RX_STOP
mlm3_3  movlw   0x01                       ;Imposta stato 1 (Ready)
        movwf   ModStat
        btfss   F_AUTPLAY                  ;Se comando Play manuale incrementa contatore
        goto    mlm_e                      ;per esecuzione prossima traccia (fino a 30)
        movf    CntPlay,w
        xorlw   D'30'
        btfsc   STATUS,Z
        clrf    CntPlay
        incf    CntPlay,f
        bsf     F_CMD_EXE
        goto    mlm_e

mlm4_1  movf    ModStat,w                  ;** Modulo in stato 4 (WaitRxCancel)
        xorlw   0x04
        btfss   STATUS,Z
        goto    mlm5_1
        movlw   D'11'                      ;Visualizza sul display '--' lampeggiante
        movwf   DigitH
        movwf   DigitL
        bsf     F_DIGBLK
        goto    mlm3_2

mlm5_1  movlw   D'12'                      ;** Modulo in stato 5 (Error)
        movwf   DigitH                     ;Visualizza sul display 'Er' fisso
        movlw   D'13'
        movwf   DigitL
        bcf     F_DIGBLK

mlm_e   btfss   F_RX_NOSD                  ;Se ricevuta risposta 'errore SD'
        goto    mlm_h                      ;imposta lo stato 5 (Error)
        bcf     F_RX_NOSD                  ;Solo per modulo VS1000
        movlw   0x05
        movwf   ModStat

mlm_h   call    drive_hb_led               ;Pilota il led HeartBeat secondo lo stato del modulo
        call    delay_100ms                ;Attesa 100ms
        goto    main_loop_input


;************************************************************************
;Inclusione Routine Mainloop e Tabelle Rom
        INCLUDE "MAIN_EXT.INC"             ;Routine per Test funzionamento
        INCLUDE "TABLES.INC"               ;Tabelle rom

        END                                ;Fine programma

