;---------------------------------------------------------------------------------
;Inizializza le variabili di ingresso matrice per rilevare correttamente
;i fronti positivi 1-->0 dall'accensione del circuito, nel caso che alcuni 
;ingressi gi si trovano a livello positivo.

; Usate:  Count1
; Output: MaInput(),...

init_matrix_input
        movlw   D'12'                    ;Richiama 12 volte la subroutine
        movwf   Count1
imi_1   call    read_matrix_input_edge   ;Legge i 30 ingressi matrice
        call    delay_100ms              ;Ritardo 100ms
        call    drive_hb_led             ;Inverte led heartbeat
        decfsz  Count1,f
        goto    imi_1
;        clrf    MaInEdge                 ;Azzera eventuali fronti rilevati
        return


;---------------------------------------------------------------------------------
;Legge i 30 ingressi su matrice e verifica i fronti 0-->1 confrontando
;lo stato degli ingressi attuali con lo stato degli ingressi precedenti.
;La matrice  composta da 5 colonne (ingressi portA) e da 6 righe (contatore CMaLine)
;Viene eseguita ogni 100ms per leggere ingressi con tensione variabile (carica e
;scarica dei condensatori sui 5 ingressi Trigger della portaA)
;La variabile di uscita MaInEdge sar 0 se non sono stati rilevati fronti
;Se rilevato fronte 0-->1 MaInEdge =1..30 (numero ingresso)
;Se rilevati due fronti 0-->1 contemporaneamente viene data priorit all'ingresso pi basso

; Input:  MaInput(), MaInPrev()
; Output: MaInEdge, MaInput(), MaInPrev()
; Usate:  CMaDelay, CMaLine, CMaInput, CMaByte, CMaBit

read_matrix_input_edge
        incf    CMaDelay,f               ;Incrementa contatore per eseguire la routine
        movf    CMaDelay,w               ;ogni 200ms
        xorlw   0x02
        btfss   STATUS,Z
        goto    rmce_1
        clrf    CMaDelay

        movlw   MaInput1+0x00            ;Preleva ingressi (singola riga) e scrive
        addwf   CMaLine,w                ;su 6 variabili MaInput() secondo contatore
        movwf   FSR
        movf    PORTA,w
        movwf   TempM
        rlf     TempM,f
        rlf     TempM,w
        andlw   B'11111000'
        movwf   INDF

        incf    CMaLine,f                ;Incrementa contatore Riga matrice fino a 6
        movf    CMaLine,w                ;Se =6 effettua confronto per rilevare fronti 0-->1
        xorlw   0x06
        btfsc   STATUS,Z
        goto    rmce_2
rmce_1  clrf    MaInEdge                 ;Se diverso, azzera variabile di uscita ed esce
        return

rmce_2  clrf    CMaLine                  ;Azzera contatore Righe
        movlw   0x01                     ;Imposta contatori per verifica fronti
        movwf   CMaInput
        clrf    CMaByte
rmce_3  movlw   0x05
        movwf   CMaBit
        movlw   MaInput1+0x00            ;Individua byte stato ingresso corrente
        addwf   CMaByte,w                ;e lo copia su TempM
        movwf   FSR
        movf    INDF,w
        movwf   TempM
        movlw   MaInPrev1+0x00           ;Individua byte stato ingresso precedente
        addwf   CMaByte,w
        movwf   FSR

rmce_4  btfsc   INDF,7                   ;Verifica fronte 0-->1
        goto    rmce_5
        btfss   TempM,7
        goto    rmce_5
        movf    CMaInput,w               ;Se rilevato fronte, assegna variabile di uscita e salta
        movwf   MaInEdge
        goto    rmce_6
rmce_5  rlf     INDF,f                   ;Ruota variabili per prossimo confronto
        rlf     TempM,f
        incf    CMaInput,f
        decfsz  CMaBit,f                 ;Decrementa numero di bit
        goto    rmce_4
        incf    CMaByte,f                ;Incrementa contatore byte fino a 6
        movf    CMaByte,w
        xorlw   0x06
        btfss   STATUS,Z
        goto    rmce_3
        clrf    MaInEdge

rmce_6  clrf    CMaByte                  ;Copia 6 variabili MaInput(x) su MaInPrev(x)
rmce_7  movlw   MaInput1+0x00
        addwf   CMaByte,w
        movwf   FSR
        movf    INDF,w
        movwf   TempM
        movlw   MaInPrev1+0x00
        addwf   CMaByte,w
        movwf   FSR
        movf    TempM,w
        movwf   INDF
        incf    CMaByte,f
        movf    CMaByte,w
        xorlw   0x06
        btfss   STATUS,Z
        goto    rmce_7
        return


;---------------------------------------------------------------------------------
;Conversione Numero binario in W in 2 cifre decimali (da 00 a 99)
; Input:  W
; Output: DigD_H, DigD_L
; Used:   TempM

conv_dec_2digit
	movwf	TempM		;Salva W in TempM
	clrf	DigD_H          ;Azzera cifre decimali
	clrf	DigD_L

cdd_3	movlw	D'10'           ;Carica 10 in w
	subwf	TempM,f         ;TempM = TempM - w
	btfss	STATUS,C	;Verifica se negativo
	goto	cdd_4
	incf	DigD_H,f        ;Incrementa digit H
	goto	cdd_3
cdd_4	movlw	D'10'		;w = TempM + 10
	addwf	TempM,w

	addwf	DigD_L,f	;Calcola digit L
	return


;---------------------------------------------------------------------------------
;Conversione Numero binario in W in 2 cifre esadecimali (da 00 a FF)
; Input:  W
; Output: DigD_H, DigD_L
; Used:   TempM

conv_hex_2digit
	movwf	 TempM          ;Salva w su appoggio

	andlw	 B'11110000'	;Converte cifra H
	movwf	 DigD_H
	swapf	 DigD_H,f
	movlw	 D'10'
	subwf	 DigD_H,w       ;w = Valore - 10
	btfss	 STATUS,C	;Verifica se >= 10 (numero 0..9 o lettera A..F)
	goto	 chd_1
	addlw	 'A'
	goto	 chd_2
chd_1	movf	 DigD_H,w
	addlw	 '0'
chd_2	movwf    DigD_H

	movf	 TempM,w        ;Converte e cifra L
	andlw	 B'00001111'
	movwf	 DigD_L
	movlw	 D'10'
	subwf	 DigD_L,w       ;w = Valore - 10
	btfss	 STATUS,C	;Verifica se > 10 (numero 0..9 o lettera A..F)
	goto	 chd_3
	addlw	 'A'
	goto	 chd_4
chd_3	movf	 DigD_L,w
	addlw	 '0'
chd_4	movwf    DigD_L
	return


;---------------------------------------------------------------------------------
;Stampa sulla seriale il carattere contenuto in w
; Input: w

tx_char clrwdt                  ;Azzera watchdog
        btfss   PIR1,TXIF       ;Aspetta se si sta trasmettendo il carattere precedente
        goto    tx_char
        movwf   TXREG           ;Copia w nel registro di trasmissione
        return


;---------------------------------------------------------------------------------
;Inverte lo stato dell'uscita Led e del Flag HeartBeat secondo lo stato del modulo
; 1 (Ready): 10% on + 90% off
; 3 (Play ): 90% on + 10% off
; 5 (Error): off
; altri    : 50% on + 50% off (default)

; Input:  ModStat, F_HBEAT
; Output: F_HBEAT, O_LED_HB
; Usate:  CntHbLed

drive_hb_led
        incf    CntHbLed,f      ;Incrementa contatore per lampeggio variabile da 0 a 9
        movf    CntHbLed,w
        xorlw   D'10'
        btfsc   STATUS,Z
        clrf    CntHbLed

        movf    ModStat,w       ;Se stato 1 (Ready) genera lampeggio 10% on + 90% off
        xorlw   0x01
        btfss   STATUS,Z
        goto    dhl_1
        movf    CntHbLed,w
        btfss   STATUS,Z
        goto    dhl_off
        goto    dhl_on

dhl_1   movf    ModStat,w       ;Se stato 3 (Play) genera lampeggio 90% on + 10% off
        xorlw   0x03
        btfss   STATUS,Z
        goto    dhl_2
        movf    CntHbLed,w
        xorlw   0x09
        btfss   STATUS,Z
        goto    dhl_on
        goto    dhl_off

dhl_2   movf    ModStat,w       ;Se stato 5 (Error) led spento
        xorlw   0x05
        btfss   STATUS,Z
        goto    dhl_3
        goto    dhl_off

dhl_3   btfsc   F_HBEAT         ;In tutti gli altri stati genera lampeggio 50% on + 50% off
        goto    dhl_off

dhl_on  bsf     F_HBEAT         ;Setta flag
        bcf     O_LED_HB        ;Accende led
        return

dhl_off bcf     F_HBEAT         ;Resetta flag
        bsf     O_LED_HB        ;Spegne led
        return


;-----------------------------------------------------
;Effettua ritardo da 1us

delay_1us
        nop
        return


;-----------------------------------------------------
;Effettua ritardo da 100ms (senza interrupt)
; Usate: CntDel1, CntDel2

delay_100ms
        movlw   D'130'          ;Ritardo ([(15*255)+3]*130) * 0.2 = 99528us
        movwf   CntDel2
del_1   clrf    CntDel1
del_2   clrwdt                  ;Ciclo 1
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        nop
        decfsz  CntDel1,f
        goto    del_2
        decfsz  CntDel2,f       ;Ciclo 2
        goto    del_1
        return
        
